/**
   @author Shin'ichiro Nakaoka
*/

#include "LazyCaller.h"
#include <QObject>
#include <QEvent>
#include <QCoreApplication>
#include <QThread>
#include <QSemaphore>
#include <boost/bind.hpp>

using namespace std;
using namespace boost;
using namespace cnoid;

namespace cnoid {

    class CallEvent : public QEvent
    {
    public:
        CallEvent(const boost::function<void(void)>& function)
            : QEvent(QEvent::User),
              function(function),
              semaphore(0) {
        }
        CallEvent(const boost::function<void(void)>& function, QSemaphore& semaphore)
            : QEvent(QEvent::User),
              function(function),
              semaphore(&semaphore) {
        }
        CallEvent(const CallEvent& org)
            : QEvent(QEvent::User),
              function(org.function),
              semaphore(org.semaphore) {
        }
        boost::function<void(void)> function;
        QSemaphore* semaphore;
    };

    class CallEventHandler : public QObject
    {
      public:
        CallEventHandler() {
            mainThreadId = QThread::currentThreadId();
        }
        virtual bool event(QEvent* e);
        Qt::HANDLE mainThreadId;
    };
    
    CallEventHandler callEventHandler;

}


void cnoid::callLater(const boost::function<void(void)>& function, int priority)
{
    CallEvent* event = new CallEvent(function);
    QCoreApplication::postEvent(&callEventHandler, event, priority);
}


void cnoid::callSynchronously(const boost::function<void(void)>& function, int priority)
{
    if(QThread::currentThreadId() == callEventHandler.mainThreadId){
        function();
        //callLater(function, priority);
    } else {
        QSemaphore semaphore;
        QCoreApplication::postEvent(&callEventHandler, new CallEvent(function, semaphore), priority);
        semaphore.acquire(); // wait for finish
    }
}


bool CallEventHandler::event(QEvent* e)
{
    CallEvent* callEvent = dynamic_cast<CallEvent*>(e);
    if(callEvent){
        callEvent->function();
        if(callEvent->semaphore){
            callEvent->semaphore->release(); // wake up the caller process
        }
        return true;
    }
    return false;
}


LazyCaller::LazyCaller()
{
    isCallEventPending = false;
    priority = IDLE_PRIORITY_HIGH;
}


LazyCaller::LazyCaller(const boost::function<void(void)>& function, int priority)
    : function(function),
      priority(priority)
{
    isCallEventPending = false;
}


void LazyCaller::set(const boost::function<void(void)>& function, int priority)
{
    this->function = function;
    this->priority = priority;
}


void LazyCaller::setPriority(int priority)
{
    this->priority = priority;
}
        

void LazyCaller::operator()()
    
{
    if(!isCallEventPending){
        isCallEventPending = true;
        callLater(boost::bind(&LazyCaller::onRequest, this), priority);
    }
}


void LazyCaller::onRequest()
{
    isCallEventPending = false;
    function();
}
