/**
   @author Shin'ichiro Nakaoka
*/

#ifndef CNOID_BASE_SCENE_ITEM_H_INCLUDED
#define CNOID_BASE_SCENE_ITEM_H_INCLUDED

#include "Item.h"
#include <cnoid/SceneGraph>
#include <cnoid/SceneProvider>
#include "exportdecl.h"

namespace cnoid {

    class CNOID_EXPORT SceneItem : public Item, public SceneProvider
    {
      public:
        static void initialize(ExtensionManager* ext);

        SceneItem();
        SceneItem(const SceneItem& org);
        virtual ~SceneItem();

        virtual void setName(const std::string& name);
        virtual SgNode* scene();

        SgTransform* topNode() { return topNode_.get(); }
        const SgTransform* topNode() const { return topNode_.get(); }

        bool loadPCD(const std::string& filename, std::ostream& os = nullout());

      protected:
        virtual ItemPtr doDuplicate() const;
        virtual bool store(Archive& archive);
        virtual bool restore(const Archive& archive);
        virtual void doPutProperties(PutPropertyFunction& putProperty);

      private:
        SgTransformPtr topNode_;
    };

    typedef boost::intrusive_ptr<SceneItem> SceneItemPtr;
}
    
#endif
