/**
   \file
   \author Shin'ichiro Nakaoka
*/

#ifndef CNOID_BODY_BODY_LOADER_H_INCLUDED
#define CNOID_BODY_BODY_LOADER_H_INCLUDED

#include "Body.h"
#include <cnoid/SignalProxy>
#include "exportdecl.h"

namespace cnoid {

    class ModelNodeSet;
    typedef boost::shared_ptr<ModelNodeSet> ModelNodeSetPtr;
    
    class BodyLoaderImpl;
  
    class CNOID_EXPORT BodyLoader
    {
      public:
        BodyLoader();
        ~BodyLoader();

        void enableShapeLoading(bool on);

        void setDivisionNumber(int n);
        void enableTriangulation(bool on);
        void enableNormalGeneration(bool on, bool doOverwrite = false);
        void enableColdetModelCreation(bool on);

        bool load(Body& body, const std::string& filename);
        BodyPtr load(const std::string& filename);

        const std::string& errorMessage();
    
        ModelNodeSetPtr modelNodeSet();

        SignalProxy< boost::signal<void(const std::string& message)> > sigMessage();

#ifdef CNOID_BACKWARD_COMPATIBILITY
        BodyPtr loadModelFile(
            const std::string& filename,
            bool doTriangulation = true, bool doNormalGeneration = true, bool createColdetModel = true);
#endif
    
      private:
        BodyLoaderImpl* impl;
    };
}

#endif
