/**
   \file
   \author Shin'ichiro Nakaoka
*/

#ifndef CNOID_BODY_CAMERA_H_INCLUDED
#define CNOID_BODY_CAMERA_H_INCLUDED

#include "Device.h"
#include "exportdecl.h"

namespace cnoid {

    class CNOID_EXPORT Camera : public Device
    {
      public:
        Camera();
        Camera(const Camera& org, bool stateOnly = false);
        void copyStateFrom(const Camera& other);
        virtual bool copyStateFrom(const DeviceState& other);
        virtual DeviceState* cloneState();
        virtual Device* clone();
        virtual void forEachActualType(boost::function<bool(const std::type_info& type)> func);
        
        bool on() const { return on_; }
        void on(bool on) { on_ = on; }

        int imageWidth() const { return imageWidth_; }
        void setImageWidth(int w) { imageWidth_ = w; }

        int imageHeight() const { return imageHeight_; }
        void setImageHeight(int h) { imageHeight_ = h; }

        double fieldOfView() const { return fieldOfView_; }
        void setFieldOfView(double f) { fieldOfView_ = f; }

      private:
        bool on_;
        int imageWidth_;
        int imageHeight_;
        double fieldOfView_;
    };

    typedef boost::intrusive_ptr<Camera> CameraPtr;

};


#endif
