/**
   @author Shin'ichiro Nakaoka
*/

#ifndef CNOID_BODYPLUGIN_OSG_SCENE_BODY_H_INCLUDED
#define CNOID_BODYPLUGIN_OSG_SCENE_BODY_H_INCLUDED

#include <boost/dynamic_bitset.hpp>
#include <cnoid/OSGSceneObject>
#include "BodyItem.h"
#include "exportdecl.h"

namespace cnoid {

    class OSGSceneBodyImpl;

    class CNOID_EXPORT OSGSceneBody : public OSGSceneObject
    {
      public:
        OSGSceneBody(BodyItemPtr bodyItem);

        void setLinkVisibilities(const boost::dynamic_bitset<>& visibilities);

        void showCenterOfMass(bool on);
        bool isCenterOfMassVisible() const;

        void showZmp(bool on);
        bool isZmpVisible() const;

        osg::Node* linkNode(Link* link);
        Link* pointedLink();
        osg::Node* pointedLinkShapeNode();

        //! Obsolete. Please use pointedLink().
        Link* getPointedSceneLink() {
            return pointedLink();
        }

        //! Obsolete. Please use pointedLinkShapeNode().
        osg::ref_ptr<osg::Node> getPointedShapeNode() {
            return pointedLinkShapeNode();
        }
		  
      protected:

        virtual ~OSGSceneBody();
        bool createSceneLinks();

        virtual void onAttachedToScene();
        virtual void onDetachedFromScene();

        virtual bool onKeyPressEvent(const OSGSceneViewEvent& event);
        virtual bool onKeyReleaseEvent(const OSGSceneViewEvent& event);
        virtual bool onButtonPressEvent(const OSGSceneViewEvent& event);
        virtual bool onButtonReleaseEvent(const OSGSceneViewEvent& event);
        virtual bool onDoubleClickEvent(const OSGSceneViewEvent& event);
        virtual bool onPointerMoveEvent(const OSGSceneViewEvent& event);
        virtual void onPointerLeaveEvent(const OSGSceneViewEvent& event);
        virtual void onContextMenuRequest(const OSGSceneViewEvent& event, MenuManager& menuManager);
        virtual void onSceneModeChanged();
        virtual bool onUndoRequest();
        virtual bool onRedoRequest();
            
      private:

        OSGSceneBodyImpl* impl;

        friend class OSGSceneBodyImpl;
    };
            
    typedef osg::ref_ptr<OSGSceneBody> OSGSceneBodyPtr;
}
    
#endif
