/*!
  @file
  @author Shin'ichiro Nakaoka
*/

#include "ZMPSeqItem.h"
#include "BodyItem.h"
#include "BodyMotionItem.h"
#include "BodyMotionEngine.h"
#include <cnoid/ZMPSeq>
#include <boost/bind.hpp>

using namespace std;
using namespace boost;
using namespace cnoid;


namespace {

    AbstractSeqItemPtr createZMPSeqItem(AbstractSeqPtr seq)
    {
        ZMPSeqPtr zmpseq = dynamic_pointer_cast<ZMPSeq>(seq);
        return zmpseq ? new Vector3SeqItem(zmpseq) : 0;
    }


    class ZMPSeqEngine : public TimeSyncItemEngine
    {
        Vector3SeqPtr seq;
        BodyItemPtr bodyItem;

    public:
        
        ZMPSeqEngine(Vector3SeqItemPtr seqItem, BodyItemPtr bodyItem)
            : seq(seqItem->seq()), bodyItem(bodyItem)
        {
            seqItem->sigUpdated().connect(bind(&TimeSyncItemEngine::notifyUpdate, this));
        }

        virtual bool onTimeChanged(double time)
        {
            bool isValidTime = false;
            if(!seq->empty()){
                bodyItem->setZmp(seq->at(seq->clampFrameIndex(seq->frameOfTime(time), isValidTime)));
            }
            return isValidTime;
        }
    };


    TimeSyncItemEnginePtr createZMPSeqEngine(BodyItemPtr bodyItem, AbstractSeqItemPtr seqItem)
    {
        if(Vector3SeqItemPtr item = dynamic_pointer_cast<Vector3SeqItem>(seqItem)){
            return make_shared<ZMPSeqEngine>(item, bodyItem);
        }
        return TimeSyncItemEnginePtr();
    }
}


void cnoid::initializeZMPSeqItem(ExtensionManager* ext)
{
    BodyMotionItem::addExtraSeqItemFactory(ZMPSeq::key(), createZMPSeqItem);
    BodyMotionEngine::addExtraSeqEngineFactory(ZMPSeq::key(), createZMPSeqEngine);
}
